package geoip

import (
	"fmt"
	"io"
	"net/http"
	"os"
	"strings"

	geoip2 "github.com/oschwald/geoip2-golang"
)

var RootURL = "https://git.paulbsd.com/paulbsd/GeoLite.mmdb/releases/download/%s/%s"
var Version = "2022.06.16"

var dbs map[string]string = map[string]string{
	"city": fmt.Sprintf(RootURL, Version, "GeoLite2-City.mmdb"),
	"asn":  fmt.Sprintf(RootURL, Version, "GeoLite2-ASN.mmdb")}

func InitGeoIP() (citydb *geoip2.Reader, asndb *geoip2.Reader, err error) {
	citydb, err = FetchDB(dbs["city"])
	if err != nil {
		return
	}
	asndb, err = FetchDB(dbs["asn"])
	if err != nil {
		return
	}
	return
}

func FetchDB(db string) (*geoip2.Reader, error) {
	if _, err := os.Stat("geoip"); err != nil {
		os.MkdirAll("geoip", 0777)
	}

	splited := strings.Split(db, "/")
	filename := splited[len(splited)-1]
	dbpath := fmt.Sprintf("%s/%s", "geoip", filename)

	if _, err := os.Stat(dbpath); err != nil {
		resp, err := http.Get(db)
		if err != nil {
			return nil, nil
		}
		f, _ := os.Create(dbpath)
		io.Copy(f, resp.Body)
	}
	return geoip2.Open(dbpath)
}
